/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:     passwd.c                                                   *
 * Created:       2000-08-25 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2004-04-20 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2000-2004 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: passwd.c 384 2005-12-29 11:30:57Z hampa $ */


#include <lib/common.h>

#include <stdio.h>


#ifdef MISC_DOS

#include <string.h>
#include <conio.h>

int crypt_get_passwd (const char *msg, unsigned char *buf, unsigned *n)
{
	unsigned i;
	char     *tmp;

	tmp = getpass (msg);

	i = strlen (tmp);
	*n = (i <= *n) ? i : *n;

	memcpy (buf, tmp, *n);

	return (0);
}

#else

#include <unistd.h>
#include <termios.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>


int crypt_get_passwd (const char *msg, unsigned char *buf, unsigned *n)
{
	unsigned       i;
	int            fd;
	unsigned char  chr;
	struct termios tios, sios;

	fd = 0;

	if (!isatty (fd)) {
		fd = open ("/dev/tty", O_RDONLY | O_NOCTTY);
		if (fd == -1) {
			return (1);
		}
	}

	if (tcgetattr (fd, &tios)) {
		return (1);
	}

	sios = tios;

	tios.c_lflag &= ~(ICANON | ECHO);
	tios.c_cc[VMIN] = 1;
	tios.c_cc[VTIME] = 0;

	if (tcsetattr (fd, TCSANOW, &tios)) {
		return (1);
	}

	fputs (msg, stderr);
	fflush (stderr);

	i = 0;

	while (1) {
		if (read (fd, &chr, 1) < 1) {
			tcsetattr (fd, TCSANOW, &sios);
			return (1);
		}

		if (chr == '\n') {
			break;
		}
		else if (chr == 8) {
			if (i > 0) {
				i -= 1;
			}
		}
		else {
			if (i < *n) {
				buf[i] = chr;
				i += 1;
			}
		}
	}

	*n = i;

	fputc ('\n', stderr);

	tcsetattr (fd, TCSANOW, &sios);

	return (0);
}

#endif


int crypt_get_passwd_fp (FILE *fp, unsigned char *buf, unsigned *n)
{
	unsigned i;
	int      c;

	i = 0;

	while (1) {
		c = fgetc (fp);

	 if (c == EOF) {
		 break;
	 }

		if (i < *n) {
			buf[i] = c;
			i += 1;
		}
	}

	*n = i;

	return (0);
}
