/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:     md4.h                                                      *
 * Created:       2004-07-08 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2005-02-19 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2004-2005 Hampa Hug <hampa@hampa.ch>                   *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: md4.h 384 2005-12-29 11:30:57Z hampa $ */


#ifndef MISC_MD4_H
#define MISC_MD4_H 1


/*!***************************************************************************
 * @short The MD4 context
 *****************************************************************************/
typedef struct {
	unsigned long h[4];

	unsigned long cnt[2];

	unsigned char buf[64];
	unsigned      buf_i;
} md4_t;



/*!***************************************************************************
 * @short Initialize an MD4 context
 *****************************************************************************/
void md4_init (md4_t *md4);

/*!***************************************************************************
 * @short Copy an MD4 context
 *****************************************************************************/
void md4_copy (md4_t *dst, const md4_t *src);

/*!***************************************************************************
 * @short Calculate an MD4 message digest
 * @param md4 The MD4 context
 * @param buf The data
 * @param cnt The number of bytes at buf
 *
 * This function can be called multiple times to add more bytes.
 *****************************************************************************/
void md4_calc (md4_t *md4, const void *buf, unsigned long cnt);

/*!***************************************************************************
 * @short End calculating an MD4 message digest
 *
 * This function must be called after the last call to md4_calc() and
 * before getting the md4 message digest.
 *****************************************************************************/
void md4_done (md4_t *md4);

/*!***************************************************************************
 * @short Get a 32bit word of the MD4 message digest
 * @param md4 The MD4 context
 * @param i   The word index. 0 <= i <= 3.
 *****************************************************************************/
unsigned long md4_get_sum_i (md4_t *md4, unsigned i);

/*!***************************************************************************
 * @short Copy the MD4 message digest to a byte buffer
 * @param md4 The MD4 context
 * @param buf The 16 byte MD4 buffer
 *****************************************************************************/
void md4_get_sum (md4_t *md4, void *buf);

/*!***************************************************************************
 * @short Calculate the MD4 message digest of a buffer
 * @param dst The 16 byte MD4 buffer
 * @param src The source buffer
 * @param n   The size in bytes of the source buffer
 *****************************************************************************/
void md4_calc1 (void *dst, const void *src, unsigned long n);


#endif
