/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:     sha224.h                                                   *
 * Created:       2005-11-18 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2005-11-18 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2005 Hampa Hug <hampa@hampa.ch>                        *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: sha224.h 384 2005-12-29 11:30:57Z hampa $ */


#ifndef MISC_SHA224_H
#define MISC_SHA224_H 1


/*!***************************************************************************
 * @short The SHA224 context
 *****************************************************************************/
typedef struct {
	unsigned      buf_i;
	unsigned char buf[64];

	unsigned long H[8];

	unsigned long cnt[2];
} sha224_t;


/*!***************************************************************************
 * @short Initialize an SHA224 context
 *****************************************************************************/
void sha224_init (sha224_t *sha);

/*!***************************************************************************
 * @short Copy an SHA224 context
 *****************************************************************************/
void sha224_copy (sha224_t *dst, const sha224_t *src);

/*!***************************************************************************
 * @short Calculate an SHA224 sum
 * @param sha The SHA224 context
 * @param buf The data
 * @param cnt The number of bytes at buf
 *
 * This function can be called multiple times to add more bytes.
 *****************************************************************************/
void sha224_calc (sha224_t *sha, const void *buf, unsigned long cnt);

/*!***************************************************************************
 * @short End calculating an SHA224 sum
 * @param sha The SHA224 context
 *
 * This function must be called after the last call to sha224_calc() and
 * before getting the sha224 sum.
 *****************************************************************************/
void sha224_done (sha224_t *sha);

/*!***************************************************************************
 * @short Get a 32bit word of the SHA224 sum
 * @param sha The SHA224 context
 * @param i   The word index. 0 <= i <= 7.
 *****************************************************************************/
unsigned long sha224_get_sum_i (sha224_t *sha, unsigned i);

/*!***************************************************************************
 * @short  Copy the SHA224 sum to a byte buffer
 * @param  sha The SHA224 context
 * @param  buf The 32 byte SHA224 buffer
 * @param  max The size of the buffer
 * @return The number of bytes written to the buffer
 *****************************************************************************/
unsigned sha224_get_sum (sha224_t *sha, void *buf, unsigned max);

/*!***************************************************************************
 * @short Calculate the SHA224 sum of a buffer
 * @param dst The 32 byte SHA224 buffer
 * @param src The source buffer
 * @param n   The size in bytes of the source buffer
 *****************************************************************************/
void sha224_calc1 (void *dst, const void *src, unsigned long n);


#endif
