/*****************************************************************************
 * misc-utils                                                                *
 *****************************************************************************/

/*****************************************************************************
 * File name:     sha512.h                                                   *
 * Created:       2005-11-18 by Hampa Hug <hampa@hampa.ch>                   *
 * Last modified: 2005-11-18 by Hampa Hug <hampa@hampa.ch>                   *
 * Copyright:     (C) 2005 Hampa Hug <hampa@hampa.ch>                        *
 *****************************************************************************/

/*****************************************************************************
 * This program is free software. You can redistribute it and / or modify it *
 * under the terms of the GNU General Public License version 2 as  published *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope  that  it  will  be  useful,  but *
 * WITHOUT  ANY   WARRANTY,   without   even   the   implied   warranty   of *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  General *
 * Public License for more details.                                          *
 *****************************************************************************/

/* $Id: sha512.h 384 2005-12-29 11:30:57Z hampa $ */


#ifndef MISC_SHA512_H
#define MISC_SHA512_H 1


/*!***************************************************************************
 * @short The SHA512 context
 *****************************************************************************/
typedef struct {
	unsigned           buf_i;
	unsigned char      buf[128];

	unsigned long long H[8];

	unsigned long long cnt[2];
} sha512_t;


/*!***************************************************************************
 * @short Initialize an SHA512 context
 *****************************************************************************/
void sha512_init (sha512_t *sha);

/*!***************************************************************************
 * @short Copy an SHA512 context
 *****************************************************************************/
void sha512_copy (sha512_t *dst, const sha512_t *src);

/*!***************************************************************************
 * @short Calculate an SHA512 sum
 * @param sha The SHA512 context
 * @param buf The data
 * @param cnt The number of bytes at buf
 *
 * This function can be called multiple times to add more bytes.
 *****************************************************************************/
void sha512_calc (sha512_t *sha, const void *buf, unsigned long cnt);

/*!***************************************************************************
 * @short End calculating an SHA512 sum
 * @param sha The SHA512 context
 *
 * This function must be called after the last call to sha512_calc() and
 * before getting the sha512 sum.
 *****************************************************************************/
void sha512_done (sha512_t *sha);

/*!***************************************************************************
 * @short Get a 64bit word of the SHA512 sum
 * @param sha The SHA512 context
 * @param i   The word index. 0 <= i <= 7.
 *****************************************************************************/
unsigned long long sha512_get_sum_i (sha512_t *sha, unsigned i);

/*!***************************************************************************
 * @short  Copy the SHA512 sum to a byte buffer
 * @param  sha The SHA512 context
 * @param  buf The 64 byte SHA512 buffer
 * @param  max The size of the buffer
 * @return The number of bytes written to the buffer
 *****************************************************************************/
unsigned sha512_get_sum (sha512_t *sha, void *buf, unsigned max);

/*!***************************************************************************
 * @short Calculate the SHA512 sum of a buffer
 * @param dst The 64 byte SHA512 buffer
 * @param src The source buffer
 * @param n   The size in bytes of the source buffer
 *****************************************************************************/
void sha512_calc1 (void *dst, const void *src, unsigned long n);


#endif
