#!/bin/bash

# $Id: lssym.sh 495 2008-08-24 22:12:01Z hampa $

case "$1" in
"-?" | "--help")
	echo "lssym: list dangling symlinks"
	echo
	echo "usage: lssym [options] dir..."
	echo "  -d, --directory string  Set a directory"
	echo "  -k, --remove            Remove dangling symlinks [no]"
	echo "  -r, --recursive         List symlinks recursively [no]"
	echo "  -v, --verbose           Verbose operation [no]"
	exit 0
	;;
"--version")
	echo "lssym (misc-utils) version %MISC_VERSION_STR (%MISC_CFG_DATE %MISC_CFG_TIME)"
	echo ""
	echo "Copyright (C) 2000-2008 by Hampa Hug <hampa@hampa.ch>"

	exit 0
	;;
esac


par_verbose=0
par_recursive=0
par_remove=0


lssym_list ()
{
	echo "$1"

	if test "x$par_remove" = "x1" ; then
		rm -f "$1"
	fi
}

lssym_dir ()
{
	local dir="$1"
	local f

	dir=${dir%/}
	dir=${dir%/}
	dir=${dir#./}

	if test "x$par_verbose" = "x1" ; then
		echo "Entering $dir" >&2
	fi

	for f in "$dir"/* ; do
		if test -L "$f" ; then
			if test ! -e "$f" ; then
				lssym_list "$f"
			fi
		else
			if test -d "$f" ; then
				if test "x$par_recursive" = "x1" ; then
					lssym_dir "$f"
				fi
			fi
		fi
	done
}

cnt=0

while test $# -gt 0 ; do
	case "$1" in
	"-d" | "--directory")
		lssym_dir "$2"
		cnt=$(($cnt+1))
		shift 1
		;;
	"-k" | "--remove")
		par_remove=1
		;;
	"-r" | "--recursive")
		par_recursive=1
		;;
	"-v" | "--verbose")
		par_verbose=1
		;;
	-*)
		echo "$0: unknown option ($1)" >&2
		exit 1
		;;
	*)
		lssym_dir "$1"
		cnt=$(($cnt+1))
		;;
	esac

	shift 1
done

if test $cnt -eq 0 ; then
	lssym_dir "."
fi
